// packages/babel/src/transforms/callExpression.ts
import { types as t2 } from "@babel/core";

// packages/babel/src/utils.ts
import { types as t } from "@babel/core";
import { addNamed } from "@babel/helper-module-imports";
function invariant(cond, message) {
  if (!cond) {
    throw new Error(message);
  }
}
function registerImportMethod(path, name, moduleName = "@macaron-css/core") {
  const imports = path.scope.getProgramParent().macaronData.imports || (path.scope.getProgramParent().macaronData.imports = /* @__PURE__ */ new Map());
  if (!imports.has(`${moduleName}:${name}`)) {
    let id = addNamed(path, name, moduleName);
    imports.set(`${moduleName}:${name}`, id);
    return id;
  } else {
    let iden = imports.get(`${moduleName}:${name}`);
    return t.cloneNode(iden);
  }
}
function getNearestIdentifier(path) {
  let currentPath = path;
  while (currentPath.parentPath !== null) {
    if (currentPath.isIdentifier()) {
      return currentPath;
    }
    let id = currentPath.get("id");
    if (!Array.isArray(id)) {
      if (id.isIdentifier())
        return id;
      if (id.isArrayPattern()) {
        for (const el of id.get("elements")) {
          if (el.isIdentifier())
            return el;
        }
      }
    }
    let key = currentPath.get("key");
    if (!Array.isArray(key) && key.isIdentifier()) {
      return key;
    }
    currentPath = currentPath.parentPath;
  }
  return null;
}
var extractionAPIs = [
  "style",
  "styleVariants",
  "globalStyle",
  "createTheme",
  "createGlobalTheme",
  "createThemeContract",
  "createGlobalThemeContract",
  "assignVars",
  "createVar",
  "fallbackVar",
  "fontFace",
  "globalFontFace",
  "keyframes",
  "globalKeyframes",
  "style",
  "styleVariants",
  "globalStyle",
  "createTheme",
  "createGlobalTheme",
  "createThemeContract",
  "createGlobalThemeContract",
  "assignVars",
  "createVar",
  "fallbackVar",
  "fontFace",
  "globalFontFace",
  "keyframes",
  "globalKeyframes",
  "recipe"
];

// packages/babel/src/transforms/callExpression.ts
function transformCallExpression(callPath, _callState) {
  const callee = callPath.get("callee");
  if (!extractionAPIs.some((api) => callee.referencesImport("@macaron-css/core", api))) {
    return;
  }
  const programParent = callPath.scope.getProgramParent();
  const nearestIdentifier = getNearestIdentifier(callPath);
  const ident = nearestIdentifier ? programParent.generateUidIdentifier(`$$macaron_${nearestIdentifier.node.name}`) : programParent.generateUidIdentifier("$$macaron_identifier");
  const importedIdent = registerImportMethod(callPath, ident.name, programParent.macaronData.cssFile);
  const declaration = t2.exportNamedDeclaration(t2.variableDeclaration("var", [
    t2.variableDeclarator(ident, t2.cloneNode(callPath.node))
  ]));
  callPath.traverse({
    Expression(expressionPath, state) {
      if (!expressionPath.isIdentifier())
        return;
      const binding = callPath.scope.getBinding(expressionPath);
      if (!binding || programParent.macaronData.bindings.includes(binding.path))
        return;
      programParent.macaronData.bindings.push(binding.path);
      programParent.macaronData.nodes.push({
        type: "binding",
        node: t2.cloneNode(findRootBinding(binding.path))
      });
    }
  });
  programParent.macaronData.nodes.push({
    type: "style",
    export: declaration,
    shouldReExport: false
  });
  callPath.replaceWith(importedIdent);
}
function findRootBinding(path) {
  var _a;
  let node;
  if (!("parent" in path) || ((_a = path.parentPath) == null ? void 0 : _a.isProgram())) {
    node = path.node;
  } else {
    node = path.parent;
  }
  return t2.cloneNode(node);
}

// packages/babel/src/transforms/postprocess.ts
import { types as t3 } from "@babel/core";
import generate from "@babel/generator";
function postprocess(path, state) {
  const programParent = path.scope;
  const program = t3.program(programParent.macaronData.nodes.map((node) => node.type === "style" ? node.export : node.node));
  const cssExtract = generate(program).code;
  state.opts.result = [programParent.macaronData.cssFile, cssExtract];
}

// packages/babel/src/transforms/styledComponents.ts
import { types as t4 } from "@babel/core";
var StyledComponentsVisitor = {
  CallExpression(callPath) {
    const callee = callPath.get("callee");
    const isSolidAdapter = callee.referencesImport("@macaron-css/solid", "styled");
    const isReactAdapter = callee.referencesImport("@macaron-css/react", "styled");
    if (!isSolidAdapter && !isReactAdapter)
      return;
    const runtimeImport = isSolidAdapter ? "@macaron-css/solid/runtime" : "@macaron-css/react/runtime";
    invariant(callPath.node.arguments.length === 2, "Wrong arguments to `styled`");
    const [tag, styles] = callPath.node.arguments;
    const styledIdent = registerImportMethod(callPath, "$$styled", runtimeImport);
    const recipeIdent = registerImportMethod(callPath, "recipe", "@macaron-css/core");
    const callExpression = t4.callExpression(styledIdent, [
      t4.cloneNode(tag),
      t4.callExpression(recipeIdent, [t4.cloneNode(styles)])
    ]);
    t4.addComment(callExpression, "leading", " @__PURE__ ");
    callPath.replaceWith(callExpression);
    callPath.scope.crawl();
  }
};

// packages/babel/src/transforms/preprocess.ts
import hash from "@emotion/hash";
function preprocess(path, state) {
  state.dependentNodes = /* @__PURE__ */ new Set();
  state.styledNodes = [];
  path.scope.macaronData = {
    imports: /* @__PURE__ */ new Map(),
    cssFile: `extracted_${hash(path.toString())}.css.ts`,
    nodes: [],
    bindings: []
  };
  path.traverse(StyledComponentsVisitor, state);
}

// packages/babel/src/index.ts
function macaronBabelPlugin() {
  return {
    name: "macaron-css-babel",
    visitor: {
      Program: {
        enter: preprocess,
        exit: postprocess
      },
      CallExpression: transformCallExpression
    }
  };
}
export {
  macaronBabelPlugin
};
//# sourceMappingURL=index.mjs.map